@@ -2,6 +2,7 @@ import copy
 import numpy as np
 import os
 import pytest
+import skimage.io as io
 import tempfile
 import xarray as xr
 
@@ -40,6 +41,17 @@ def test_get_single_compartment_props():
         cell_props_columns=cell_props.columns.values
     )
 
+    # test that blank segmentation mask is handled appropriately
+    cell_props_blank = marker_quantification.get_single_compartment_props(
+        np.zeros((40, 40), dtype='int'),
+        regionprops_base,
+        regionprops_single_comp)
+
+    misc_utils.verify_same_elements(
+        all_features=copy.deepcopy(regionprops_base) + regionprops_single_comp,
+        cell_props_columns=cell_props_blank.columns.values
+    )
+
 
 def test_assign_single_compartment_props():
     cell_mask, channel_data = test_utils.create_test_extraction_data()
@@ -133,8 +145,8 @@ def test_assign_multi_compartment_features():
         compartment_names=['whole_cell', 'nuclear']
     )
 
-    # create a sample marker count matrix with 2 compartments, 3 cell ids, and 3 features
-    sample_marker_counts = np.zeros((2, 3, 3))
+    # create a sample marker count matrix with 2 compartments, 3 cell ids, and 2 features
+    sample_marker_counts = np.zeros((2, 3, 2))
 
     # cell 0: no nucleus
     sample_marker_counts[0, 0, 1] = 5
@@ -151,7 +163,7 @@ def test_assign_multi_compartment_features():
     sample_marker_counts = xr.DataArray(copy.copy(sample_marker_counts),
                                         coords=[['whole_cell', 'nuclear'],
                                                 [0, 1, 2],
-                                                ['feat_1', 'area', 'nc_ratio']],
+                                                ['feat_1', 'area']],
                                         dims=['compartments', 'cell_id', 'features'])
 
     # define the nuclear properties
@@ -161,6 +173,9 @@ def test_assign_multi_compartment_features():
         sample_marker_counts, regionprops_multi_comp
     )
 
+    # assert we added nc_ratio as a features key
+    assert 'nc_ratio' in sample_marker_counts.features.values
+
     # testing cell 0
     assert sample_marker_counts.loc['whole_cell', 0, 'nc_ratio'] == 0
     assert sample_marker_counts.loc['nuclear', 0, 'nc_ratio'] == 0
@@ -235,6 +250,14 @@ def test_compute_marker_counts_base():
         > center_extraction.loc['whole_cell', :, 'chan0'].values
     )
 
+    # blank segmentation mask results in the cells column of length 0
+    blank_labels = test_utils.make_labels_xarray(label_data=np.zeros((1, 40, 40, 1), dtype='int'),
+                                                 compartment_names=['whole_cell'])
+
+    blank_output = marker_quantification.compute_marker_counts(input_images=input_images,
+                                                               segmentation_labels=blank_labels[0])
+    assert blank_output.shape[1] == 0
+
 
 def test_compute_marker_counts_equal_masks():
     cell_mask, channel_data = test_utils.create_test_extraction_data()
@@ -438,6 +461,11 @@ def test_create_marker_count_matrices_base():
     assert np.array_equal(normalized['chan0'], np.repeat(1, len(normalized)))
     assert np.array_equal(normalized['chan1'], np.repeat(5, len(normalized)))
 
+    # blank image doesn't cause any issues
+    segmentation_labels.values[1, ...] = 0
+    _ = marker_quantification.create_marker_count_matrices(segmentation_labels,
+                                                           channel_data)
+
     # error checking
     with pytest.raises(ValueError):
         # attempt to pass non-xarray for segmentation_labels
@@ -515,6 +543,11 @@ def test_create_marker_count_matrices_multiple_compartments():
     normalized_with_nuc = normalized.loc[normalized['label'] != 2, ['label', 'label_nuclear']]
     assert np.array_equal(normalized_with_nuc['label'] * 2, normalized_with_nuc['label_nuclear'])
 
+    # blank nuclear segmentation mask doesn't cause any issues
+    segmentation_labels_unequal.values[1, ..., 1] = 0
+    _ = marker_quantification.create_marker_count_matrices(segmentation_labels_unequal,
+                                                           channel_data, nuclear_counts=True)
+
 
 def test_generate_cell_data_tree_loading():
     # is_mibitiff False case, load from directory tree
@@ -544,48 +577,70 @@ def test_generate_cell_data_tree_loading():
         fovs_subset_ext[1] = str(fovs_subset_ext[1]) + ".tiff"
 
         # generate a sample segmentation_mask
-        cell_mask, _ = test_utils.create_test_extraction_data()
+        cell_masks = np.random.randint(low=0, high=5, size=(3, 40, 40, 2), dtype="int16")
+        cell_masks[0, :, :, 0] = cell_masks[0, :, :, 0]
+        cell_masks[1, 5:, 5:, 0] = cell_masks[0, :-5, :-5, 0]
+        cell_masks[2, 10:, 10:, 0] = cell_masks[0, :-10, :-10, 0] / 2
+        cell_masks[..., 1] = cell_masks[..., 0]
 
-        cell_masks = np.zeros((3, 40, 40, 1), dtype="int16")
-        cell_masks[0, :, :, 0] = cell_mask[0, :, :, 0]
-        cell_masks[1, 5:, 5:, 0] = cell_mask[0, :-5, :-5, 0]
-        cell_masks[2, 10:, 10:, 0] = cell_mask[0, :-10, :-10, 0]
+        for fov in range(cell_masks.shape[0]):
+            fov_whole_cell = cell_masks[fov, :, :, 0]
+            fov_nuclear = cell_masks[fov, :, :, 1]
+            io.imsave(os.path.join(temp_dir, 'fov%d_feature_0.tif' % fov), fov_whole_cell,
+                      check_contrast=False)
+            io.imsave(os.path.join(temp_dir, 'fov%d_feature_1.tif' % fov), fov_nuclear,
+                      check_contrast=False)
 
-        segmentation_masks = test_utils.make_labels_xarray(
-            label_data=cell_masks,
-            compartment_names=['whole_cell']
-        )
-
-        with pytest.raises(ValueError):
+        with pytest.raises(FileNotFoundError):
             # specifying fovs not in the original segmentation mask
             marker_quantification.generate_cell_table(
-                segmentation_labels=segmentation_masks.loc[["fov1"]], tiff_dir=tiff_dir,
-                img_sub_folder=img_sub_folder, is_mibitiff=False, fovs=["fov1", "fov2"],
+                segmentation_dir=temp_dir, tiff_dir=tiff_dir,
+                img_sub_folder=img_sub_folder, is_mibitiff=False, fovs=["fov2", "fov3"],
                 batch_size=5)
 
         # generate sample norm and arcsinh data for all fovs
-        norm_data, arcsinh_data = marker_quantification.generate_cell_table(
-            segmentation_labels=segmentation_masks, tiff_dir=tiff_dir,
+        norm_data_all_fov, arcsinh_data_all_fov = marker_quantification.generate_cell_table(
+            segmentation_dir=temp_dir, tiff_dir=tiff_dir,
             img_sub_folder=img_sub_folder, is_mibitiff=False, fovs=None, batch_size=2)
 
-        assert norm_data.shape[0] > 0 and norm_data.shape[1] > 0
-        assert arcsinh_data.shape[0] > 0 and arcsinh_data.shape[1] > 0
+        assert norm_data_all_fov.shape[0] > 0 and norm_data_all_fov.shape[1] > 0
+        assert arcsinh_data_all_fov.shape[0] > 0 and arcsinh_data_all_fov.shape[1] > 0
 
         # generate sample norm and arcsinh data for a subset of fovs
-        norm_data, arcsinh_data = marker_quantification.generate_cell_table(
-            segmentation_labels=segmentation_masks, tiff_dir=tiff_dir,
+        norm_data_fov_sub, arcsinh_data_fov_sub = marker_quantification.generate_cell_table(
+            segmentation_dir=temp_dir, tiff_dir=tiff_dir,
             img_sub_folder=img_sub_folder, is_mibitiff=False, fovs=fovs_subset, batch_size=2)
 
-        assert norm_data.shape[0] > 0 and norm_data.shape[1] > 0
-        assert arcsinh_data.shape[0] > 0 and arcsinh_data.shape[1] > 0
+        assert norm_data_fov_sub.shape[0] > 0 and norm_data_fov_sub.shape[1] > 0
+        assert arcsinh_data_fov_sub.shape[0] > 0 and arcsinh_data_fov_sub.shape[1] > 0
 
-        # generate sample norm and arcsinh data for a subset of fovs
-        norm_data, arcsinh_data = marker_quantification.generate_cell_table(
-            segmentation_labels=segmentation_masks, tiff_dir=tiff_dir,
+        # generate sample norm and arcsinh data for a subset of fovs with extensions
+        norm_data_fov_ext, arcsinh_data_fov_ext = marker_quantification.generate_cell_table(
+            segmentation_dir=temp_dir, tiff_dir=tiff_dir,
             img_sub_folder=img_sub_folder, is_mibitiff=False, fovs=fovs_subset_ext, batch_size=2)
 
-        assert norm_data.shape[0] > 0 and norm_data.shape[1] > 0
-        assert arcsinh_data.shape[0] > 0 and arcsinh_data.shape[1] > 0
+        assert norm_data_fov_ext.shape[0] > 0 and norm_data_fov_ext.shape[1] > 0
+        assert arcsinh_data_fov_ext.shape[0] > 0 and arcsinh_data_fov_ext.shape[1] > 0
+
+        # test nuclear_counts True
+        norm_data_nuc, arcsinh_data_nuc = marker_quantification.generate_cell_table(
+            segmentation_dir=temp_dir, tiff_dir=tiff_dir,
+            img_sub_folder=img_sub_folder, is_mibitiff=False, fovs=fovs_subset,
+            batch_size=2, nuclear_counts=True)
+
+        assert norm_data_nuc.shape[0] == norm_data_fov_sub.shape[0]
+        assert norm_data_nuc.shape[1] == norm_data_fov_sub.shape[1] * 2 + 1
+        misc_utils.verify_in_list(
+            nuclear_col='nc_ratio',
+            nuc_cell_table_cols=norm_data_nuc.columns.values
+        )
+
+        assert arcsinh_data_nuc.shape[0] == arcsinh_data_fov_sub.shape[0]
+        assert arcsinh_data_nuc.shape[1] == norm_data_fov_sub.shape[1] * 2 + 1
+        misc_utils.verify_in_list(
+            nuclear_col='nc_ratio',
+            nuc_cell_table_cols=norm_data_nuc.columns.values
+        )
 
 
 def test_generate_cell_data_mibitiff_loading():
@@ -615,39 +670,63 @@ def test_generate_cell_data_mibitiff_loading():
         )
 
         # generate a sample segmentation_mask
-        cell_mask, _ = test_utils.create_test_extraction_data()
-        cell_masks = np.zeros((3, 40, 40, 1), dtype="int16")
-        cell_masks[0, :, :, 0] = cell_mask[0, :, :, 0]
-        cell_masks[1, 5:, 5:, 0] = cell_mask[0, :-5, :-5, 0]
-        cell_masks[2, 10:, 10:, 0] = cell_mask[0, :-10, :-10, 0]
-        segmentation_masks = test_utils.make_labels_xarray(
-            label_data=cell_masks,
-            compartment_names=['whole_cell']
-        )
+        cell_masks = np.random.randint(low=0, high=5, size=(3, 40, 40, 2), dtype="int16")
+        cell_masks[0, :, :, 0] = cell_masks[0, :, :, 0]
+        cell_masks[1, 5:, 5:, 0] = cell_masks[0, :-5, :-5, 0]
+        cell_masks[2, 10:, 10:, 0] = cell_masks[0, :-10, :-10, 0]
+        cell_masks[..., 1] = cell_masks[..., 0]
+
+        for fov in range(cell_masks.shape[0]):
+            fov_whole_cell = cell_masks[fov, :, :, 0]
+            fov_nuclear = cell_masks[fov, :, :, 1]
+            io.imsave(os.path.join(temp_dir, 'fov%d_feature_0.tif' % fov), fov_whole_cell,
+                      check_contrast=False)
+            io.imsave(os.path.join(temp_dir, 'fov%d_feature_1.tif' % fov), fov_nuclear,
+                      check_contrast=False)
 
         # generate sample norm and arcsinh data for all fovs
-        norm_data, arcsinh_data = marker_quantification.generate_cell_table(
-            segmentation_labels=segmentation_masks, tiff_dir=tiff_dir,
+        norm_data_all_fov, arcsinh_data_all_fov = marker_quantification.generate_cell_table(
+            segmentation_dir=temp_dir, tiff_dir=tiff_dir,
             img_sub_folder=tiff_dir, is_mibitiff=True, fovs=None, batch_size=2)
 
-        assert norm_data.shape[0] > 0 and norm_data.shape[1] > 0
-        assert arcsinh_data.shape[0] > 0 and arcsinh_data.shape[1] > 0
+        assert norm_data_all_fov.shape[0] > 0 and norm_data_all_fov.shape[1] > 0
+        assert arcsinh_data_all_fov.shape[0] > 0 and arcsinh_data_all_fov.shape[1] > 0
 
         # generate sample norm and arcsinh data for a subset of fovs
-        norm_data, arcsinh_data = marker_quantification.generate_cell_table(
-            segmentation_labels=segmentation_masks, tiff_dir=tiff_dir,
+        norm_data_fov_sub, arcsinh_data_fov_sub = marker_quantification.generate_cell_table(
+            segmentation_dir=temp_dir, tiff_dir=tiff_dir,
             img_sub_folder=tiff_dir, is_mibitiff=True, fovs=fovs_subset, batch_size=2)
 
-        assert norm_data.shape[0] > 0 and norm_data.shape[1] > 0
-        assert arcsinh_data.shape[0] > 0 and arcsinh_data.shape[1] > 0
+        assert norm_data_fov_sub.shape[0] > 0 and norm_data_fov_sub.shape[1] > 0
+        assert arcsinh_data_fov_sub.shape[0] > 0 and arcsinh_data_fov_sub.shape[1] > 0
 
-        # generate sample norm and arcsinh for a subset of fovs with file extensions
-        norm_data, arcsinh_data = marker_quantification.generate_cell_table(
-            segmentation_labels=segmentation_masks, tiff_dir=tiff_dir,
+        # generate sample norm and arcsinh data for a subset of fovs with extensions
+        norm_data_fov_ext, arcsinh_data_fov_ext = marker_quantification.generate_cell_table(
+            segmentation_dir=temp_dir, tiff_dir=tiff_dir,
             img_sub_folder=tiff_dir, is_mibitiff=True, fovs=fovs_subset_ext, batch_size=2)
 
-        assert norm_data.shape[0] > 0 and norm_data.shape[1] > 0
-        assert arcsinh_data.shape[0] > 0 and arcsinh_data.shape[1] > 0
+        assert norm_data_fov_ext.shape[0] > 0 and norm_data_fov_ext.shape[1] > 0
+        assert arcsinh_data_fov_ext.shape[0] > 0 and arcsinh_data_fov_ext.shape[1] > 0
+
+        # test nuclear_counts True
+        norm_data_nuc, arcsinh_data_nuc = marker_quantification.generate_cell_table(
+            segmentation_dir=temp_dir, tiff_dir=tiff_dir,
+            img_sub_folder=tiff_dir, is_mibitiff=True, fovs=fovs_subset,
+            batch_size=2, nuclear_counts=True)
+
+        assert norm_data_nuc.shape[0] == norm_data_fov_sub.shape[0]
+        assert norm_data_nuc.shape[1] == norm_data_fov_sub.shape[1] * 2 + 1
+        misc_utils.verify_in_list(
+            nuclear_col='nc_ratio',
+            nuc_cell_table_cols=norm_data_nuc.columns.values
+        )
+
+        assert arcsinh_data_nuc.shape[0] == arcsinh_data_fov_sub.shape[0]
+        assert arcsinh_data_nuc.shape[1] == norm_data_fov_sub.shape[1] * 2 + 1
+        misc_utils.verify_in_list(
+            nuclear_col='nc_ratio',
+            nuc_cell_table_cols=norm_data_nuc.columns.values
+        )
 
 
 def test_generate_cell_data_extractions():
@@ -670,20 +749,22 @@ def test_generate_cell_data_extractions():
         )
 
         # generate a sample segmentation_mask
-        cell_mask, _ = test_utils.create_test_extraction_data()
+        cell_masks = np.random.randint(low=0, high=5, size=(3, 40, 40, 2), dtype="int16")
+        cell_masks[0, :, :, 0] = cell_masks[0, :, :, 0]
+        cell_masks[1, 5:, 5:, 0] = cell_masks[0, :-5, :-5, 0]
+        cell_masks[2, 10:, 10:, 0] = cell_masks[0, :-10, :-10, 0]
+        cell_masks[..., 1] = cell_masks[..., 0]
 
-        cell_masks = np.zeros((3, 40, 40, 1), dtype="int16")
-        cell_masks[0, :, :, 0] = cell_mask[0, :, :, 0]
-        cell_masks[1, 5:, 5:, 0] = cell_mask[0, :-5, :-5, 0]
-        cell_masks[2, 10:, 10:, 0] = cell_mask[0, :-10, :-10, 0]
-
-        segmentation_masks = test_utils.make_labels_xarray(
-            label_data=cell_masks,
-            compartment_names=['whole_cell']
-        )
+        for fov in range(cell_masks.shape[0]):
+            fov_whole_cell = cell_masks[fov, :, :, 0]
+            fov_nuclear = cell_masks[fov, :, :, 1]
+            io.imsave(os.path.join(temp_dir, 'fov%d_feature_0.tif' % fov), fov_whole_cell,
+                      check_contrast=False)
+            io.imsave(os.path.join(temp_dir, 'fov%d_feature_1.tif' % fov), fov_nuclear,
+                      check_contrast=False)
 
         default_norm_data, _ = marker_quantification.generate_cell_table(
-            segmentation_labels=segmentation_masks, tiff_dir=tiff_dir,
+            segmentation_dir=temp_dir, tiff_dir=tiff_dir,
             img_sub_folder=img_sub_folder, is_mibitiff=False, batch_size=2,
         )
 
@@ -703,10 +784,26 @@ def test_generate_cell_data_extractions():
 
         # verify thresh kwarg passes through
         positive_pixel_data, _ = marker_quantification.generate_cell_table(
-            segmentation_labels=segmentation_masks, tiff_dir=tiff_dir,
+            segmentation_dir=temp_dir, tiff_dir=tiff_dir,
             img_sub_folder=img_sub_folder, is_mibitiff=False, batch_size=2,
             extraction='positive_pixel', **thresh_kwargs
         )
 
         assert np.all(positive_pixel_data.iloc[:4][['chan0', 'chan1']].values == 0)
         assert np.all(positive_pixel_data.iloc[4:][chans].values == 1)
+
+        # verify thresh kwarg passes through and nuclear counts True
+        positive_pixel_data_nuc, _ = marker_quantification.generate_cell_table(
+            segmentation_dir=temp_dir, tiff_dir=tiff_dir,
+            img_sub_folder=img_sub_folder, is_mibitiff=False, batch_size=2,
+            extraction='positive_pixel', nuclear_counts=True, **thresh_kwargs
+        )
+
+        assert np.all(positive_pixel_data_nuc.iloc[:4][['chan0', 'chan1']].values == 0)
+        assert np.all(positive_pixel_data_nuc.iloc[4:][chans].values == 1)
+        assert positive_pixel_data_nuc.shape[0] == positive_pixel_data.shape[0]
+        assert positive_pixel_data_nuc.shape[1] == positive_pixel_data.shape[1] * 2 + 1
+        misc_utils.verify_in_list(
+            nuclear_col='nc_ratio',
+            nuc_cell_table_cols=positive_pixel_data_nuc.columns.values
+        )
